@@ -143,44 +143,61 @@ static sqfs_s32 lzma_uncomp_block(sqfs_compressor_t *base, const sqfs_u8 *in,
 	return hdrsize;
 }
 
-static sqfs_compressor_t *lzma_create_copy(sqfs_compressor_t *cmp)
+static void lzma_get_configuration(const sqfs_compressor_t *base,
+				   sqfs_compressor_config_t *cfg)
+{
+	const lzma_compressor_t *lzma = (const lzma_compressor_t *)base;
+
+	memset(cfg, 0, sizeof(*cfg));
+	cfg->id = SQFS_COMP_LZMA;
+	cfg->block_size = lzma->block_size;
+
+	if (base->do_block == lzma_uncomp_block)
+		cfg->flags |= SQFS_COMP_FLAG_UNCOMPRESS;
+}
+
+static sqfs_object_t *lzma_create_copy(const sqfs_object_t *cmp)
 {
 	lzma_compressor_t *copy = malloc(sizeof(*copy));
 
 	if (copy != NULL)
 		memcpy(copy, cmp, sizeof(*copy));
 
-	return (sqfs_compressor_t *)copy;
+	return (sqfs_object_t *)copy;
 }
 
-static void lzma_destroy(sqfs_compressor_t *base)
+static void lzma_destroy(sqfs_object_t *base)
 {
 	free(base);
 }
 
-sqfs_compressor_t *lzma_compressor_create(const sqfs_compressor_config_t *cfg)
+int lzma_compressor_create(const sqfs_compressor_config_t *cfg,
+			   sqfs_compressor_t **out)
 {
 	sqfs_compressor_t *base;
 	lzma_compressor_t *lzma;
 
 	if (cfg->flags & ~SQFS_COMP_FLAG_GENERIC_ALL)
-		return NULL;
+		return SQFS_ERROR_UNSUPPORTED;
 
 	lzma = calloc(1, sizeof(*lzma));
 	base = (sqfs_compressor_t *)lzma;
 	if (lzma == NULL)
-		return NULL;
+		return SQFS_ERROR_ALLOC;
 
 	lzma->block_size = cfg->block_size;
 
 	if (lzma->block_size < SQFS_META_BLOCK_SIZE)
 		lzma->block_size = SQFS_META_BLOCK_SIZE;
 
-	base->destroy = lzma_destroy;
+	base->get_configuration = lzma_get_configuration;
 	base->do_block = (cfg->flags & SQFS_COMP_FLAG_UNCOMPRESS) ?
 		lzma_uncomp_block : lzma_comp_block;
 	base->write_options = lzma_write_options;
 	base->read_options = lzma_read_options;
-	base->create_copy = lzma_create_copy;
-	return base;
+	((sqfs_object_t *)base)->copy = lzma_create_copy;
+	((sqfs_object_t *)base)->destroy = lzma_destroy;
+
+	*out = base;
+	return 0;
 }
