@@ -196,7 +196,22 @@ static sqfs_s32 xz_uncomp_block(sqfs_compressor_t *base, const sqfs_u8 *in,
 	return SQFS_ERROR_COMPRESSOR;
 }
 
-static sqfs_compressor_t *xz_create_copy(sqfs_compressor_t *cmp)
+static void xz_get_configuration(const sqfs_compressor_t *base,
+				 sqfs_compressor_config_t *cfg)
+{
+	const xz_compressor_t *xz = (const xz_compressor_t *)base;
+
+	memset(cfg, 0, sizeof(*cfg));
+	cfg->id = SQFS_COMP_XZ;
+	cfg->flags = xz->flags;
+	cfg->block_size = xz->block_size;
+	cfg->opt.xz.dict_size = xz->dict_size;
+
+	if (base->do_block == xz_uncomp_block)
+		cfg->flags |= SQFS_COMP_FLAG_UNCOMPRESS;
+}
+
+static sqfs_object_t *xz_create_copy(const sqfs_object_t *cmp)
 {
 	xz_compressor_t *xz = malloc(sizeof(*xz));
 
@@ -204,40 +219,44 @@ static sqfs_compressor_t *xz_create_copy(sqfs_compressor_t *cmp)
 		return NULL;
 
 	memcpy(xz, cmp, sizeof(*xz));
-	return (sqfs_compressor_t *)xz;
+	return (sqfs_object_t *)xz;
 }
 
-static void xz_destroy(sqfs_compressor_t *base)
+static void xz_destroy(sqfs_object_t *base)
 {
 	free(base);
 }
 
-sqfs_compressor_t *xz_compressor_create(const sqfs_compressor_config_t *cfg)
+int xz_compressor_create(const sqfs_compressor_config_t *cfg,
+			 sqfs_compressor_t **out)
 {
 	sqfs_compressor_t *base;
 	xz_compressor_t *xz;
 
 	if (cfg->flags & ~(SQFS_COMP_FLAG_GENERIC_ALL |
 			   SQFS_COMP_FLAG_XZ_ALL)) {
-		return NULL;
+		return SQFS_ERROR_UNSUPPORTED;
 	}
 
 	if (!is_dict_size_valid(cfg->opt.xz.dict_size))
-		return NULL;
+		return SQFS_ERROR_UNSUPPORTED;
 
 	xz = calloc(1, sizeof(*xz));
 	base = (sqfs_compressor_t *)xz;
 	if (xz == NULL)
-		return NULL;
+		return SQFS_ERROR_ALLOC;
 
 	xz->flags = cfg->flags;
 	xz->dict_size = cfg->opt.xz.dict_size;
 	xz->block_size = cfg->block_size;
-	base->destroy = xz_destroy;
+	base->get_configuration = xz_get_configuration;
 	base->do_block = (cfg->flags & SQFS_COMP_FLAG_UNCOMPRESS) ?
 		xz_uncomp_block : xz_comp_block;
 	base->write_options = xz_write_options;
 	base->read_options = xz_read_options;
-	base->create_copy = xz_create_copy;
-	return base;
+	((sqfs_object_t *)base)->copy = xz_create_copy;
+	((sqfs_object_t *)base)->destroy = xz_destroy;
+
+	*out = base;
+	return 0;
 }
