@@ -18,6 +18,7 @@
 
 typedef struct {
 	sqfs_compressor_t base;
+	size_t block_size;
 	ZSTD_CCtx *zctx;
 	int level;
 } zstd_compressor_t;
@@ -91,7 +92,22 @@ static sqfs_s32 zstd_uncomp_block(sqfs_compressor_t *base, const sqfs_u8 *in,
 	return ret;
 }
 
-static sqfs_compressor_t *zstd_create_copy(sqfs_compressor_t *cmp)
+static void zstd_get_configuration(const sqfs_compressor_t *base,
+				   sqfs_compressor_config_t *cfg)
+{
+	const zstd_compressor_t *zstd = (const zstd_compressor_t *)base;
+
+	memset(cfg, 0, sizeof(*cfg));
+	cfg->id = SQFS_COMP_ZSTD;
+
+	cfg->block_size = zstd->block_size;
+	cfg->opt.zstd.level = zstd->level;
+
+	if (base->do_block == zstd_uncomp_block)
+		cfg->flags |= SQFS_COMP_FLAG_UNCOMPRESS;
+}
+
+static sqfs_object_t *zstd_create_copy(const sqfs_object_t *cmp)
 {
 	zstd_compressor_t *zstd = malloc(sizeof(*zstd));
 
@@ -107,10 +123,10 @@ static sqfs_compressor_t *zstd_create_copy(sqfs_compressor_t *cmp)
 		return NULL;
 	}
 
-	return (sqfs_compressor_t *)zstd;
+	return (sqfs_object_t *)zstd;
 }
 
-static void zstd_destroy(sqfs_compressor_t *base)
+static void zstd_destroy(sqfs_object_t *base)
 {
 	zstd_compressor_t *zstd = (zstd_compressor_t *)base;
 
@@ -118,35 +134,40 @@ static void zstd_destroy(sqfs_compressor_t *base)
 	free(zstd);
 }
 
-sqfs_compressor_t *zstd_compressor_create(const sqfs_compressor_config_t *cfg)
+int zstd_compressor_create(const sqfs_compressor_config_t *cfg,
+			   sqfs_compressor_t **out)
 {
 	zstd_compressor_t *zstd;
 	sqfs_compressor_t *base;
 
 	if (cfg->flags & ~SQFS_COMP_FLAG_GENERIC_ALL)
-		return NULL;
+		return SQFS_ERROR_UNSUPPORTED;
 
 	if (cfg->opt.zstd.level < 1 ||
 	    cfg->opt.zstd.level > ZSTD_maxCLevel()) {
-		return NULL;
+		return SQFS_ERROR_UNSUPPORTED;
 	}
 
 	zstd = calloc(1, sizeof(*zstd));
 	base = (sqfs_compressor_t *)zstd;
 	if (zstd == NULL)
-		return NULL;
+		return SQFS_ERROR_ALLOC;
 
+	zstd->block_size = cfg->block_size;
 	zstd->zctx = ZSTD_createCCtx();
 	if (zstd->zctx == NULL) {
 		free(zstd);
-		return NULL;
+		return SQFS_ERROR_COMPRESSOR;
 	}
 
-	base->destroy = zstd_destroy;
+	base->get_configuration = zstd_get_configuration;
 	base->do_block = cfg->flags & SQFS_COMP_FLAG_UNCOMPRESS ?
 		zstd_uncomp_block : zstd_comp_block;
 	base->write_options = zstd_write_options;
 	base->read_options = zstd_read_options;
-	base->create_copy = zstd_create_copy;
-	return base;
+	((sqfs_object_t *)base)->copy = zstd_create_copy;
+	((sqfs_object_t *)base)->destroy = zstd_destroy;
+
+	*out = base;
+	return 0;
 }
