@@ -20,6 +20,8 @@
 #include <stdlib.h>
 
 struct sqfs_dir_reader_t {
+	sqfs_object_t base;
+
 	sqfs_meta_reader_t *meta_dir;
 	sqfs_meta_reader_t *meta_inode;
 	const sqfs_super_t *super;
@@ -34,6 +36,41 @@ struct sqfs_dir_reader_t {
 	sqfs_u16 inode_offset;
 };
 
+static void dir_reader_destroy(sqfs_object_t *obj)
+{
+	sqfs_dir_reader_t *rd = (sqfs_dir_reader_t *)obj;
+
+	sqfs_destroy(rd->meta_inode);
+	sqfs_destroy(rd->meta_dir);
+	free(rd);
+}
+
+static sqfs_object_t *dir_reader_copy(const sqfs_object_t *obj)
+{
+	const sqfs_dir_reader_t *rd = (const sqfs_dir_reader_t *)obj;
+	sqfs_dir_reader_t *copy  = malloc(sizeof(*copy));
+
+	if (copy == NULL)
+		return NULL;
+
+	memcpy(copy, rd, sizeof(*copy));
+
+	copy->meta_inode = sqfs_copy(rd->meta_inode);
+	if (copy->meta_inode == NULL)
+		goto fail_mino;
+
+	copy->meta_dir = sqfs_copy(rd->meta_dir);
+	if (copy->meta_dir == NULL)
+		goto fail_mdir;
+
+	return (sqfs_object_t *)copy;
+fail_mdir:
+	sqfs_destroy(copy->meta_inode);
+fail_mino:
+	free(copy);
+	return NULL;
+}
+
 sqfs_dir_reader_t *sqfs_dir_reader_create(const sqfs_super_t *super,
 					  sqfs_compressor_t *cmp,
 					  sqfs_file_t *file)
@@ -66,22 +103,17 @@ sqfs_dir_reader_t *sqfs_dir_reader_create(const sqfs_super_t *super,
 	rd->meta_dir = sqfs_meta_reader_create(file, cmp, start, limit);
 
 	if (rd->meta_dir == NULL) {
-		sqfs_meta_reader_destroy(rd->meta_inode);
+		sqfs_destroy(rd->meta_inode);
 		free(rd);
 		return NULL;
 	}
 
+	((sqfs_object_t *)rd)->destroy = dir_reader_destroy;
+	((sqfs_object_t *)rd)->copy = dir_reader_copy;
 	rd->super = super;
 	return rd;
 }
 
-void sqfs_dir_reader_destroy(sqfs_dir_reader_t *rd)
-{
-	sqfs_meta_reader_destroy(rd->meta_inode);
-	sqfs_meta_reader_destroy(rd->meta_dir);
-	free(rd);
-}
-
 int sqfs_dir_reader_open_dir(sqfs_dir_reader_t *rd,
 			     const sqfs_inode_generic_t *inode)
 {
@@ -210,15 +242,28 @@ int sqfs_dir_reader_get_root_inode(sqfs_dir_reader_t *rd,
 					   block_start, offset, inode);
 }
 
-int sqfs_dir_reader_find_by_path(sqfs_dir_reader_t *rd, const char *path,
-				 sqfs_inode_generic_t **out)
+int sqfs_dir_reader_find_by_path(sqfs_dir_reader_t *rd,
+				 const sqfs_inode_generic_t *start,
+				 const char *path, sqfs_inode_generic_t **out)
 {
 	sqfs_inode_generic_t *inode;
 	sqfs_dir_entry_t *ent;
 	const char *ptr;
-	int ret;
+	int ret = 0;
+
+	if (start == NULL) {
+		ret = sqfs_dir_reader_get_root_inode(rd, &inode);
+	} else {
+		inode = alloc_flex(sizeof(*inode), 1,
+				   start->payload_bytes_used);
+		if (inode == NULL) {
+			ret = SQFS_ERROR_ALLOC;
+		} else {
+			memcpy(inode, start,
+			       sizeof(*start) + start->payload_bytes_used);
+		}
+	}
 
-	ret = sqfs_dir_reader_get_root_inode(rd, &inode);
 	if (ret)
 		return ret;
 
