@@ -13,7 +13,7 @@
 #include <string.h>
 
 int sqfs_super_init(sqfs_super_t *super, size_t block_size, sqfs_u32 mtime,
-		    E_SQFS_COMPRESSOR compressor)
+		    SQFS_COMPRESSOR compressor)
 {
 	unsigned int i;
 
